local ck = require "resty.cookie"
local cjson = require "cjson.safe"
local redis = require "resty.redis"

local _M = {}

function _M.get_jasima_host()
    local headers = ngx.req.get_headers()
    if headers["Jasima-Host"] then
        return headers["Jasima-Host"]
    end

    local cookie, err = ck:new()
    if not cookie then
        ngx.log(ngx.ERR, "failed to get cookie: ", err)
        return nil
    end

    local jasima_cookie, err = cookie:get("jasima_host")
    if jasima_cookie then
        return jasima_cookie
    elseif err then
        ngx.log(ngx.ERR, "failed to get jasima_host cookie: ", err)
    end

    return nil
end

function _M.get_jasima_pool()
    local headers = ngx.req.get_headers()
    if headers["Jasima-Pool"] then
        return headers["Jasima-Pool"]
    end

    local cookie, err = ck:new()
    if not cookie then
        ngx.log(ngx.ERR, "failed to get cookie: ", err)
        return nil
    end

    local jasima_cookie, err = cookie:get("jasima_pool")
    if jasima_cookie then
        return jasima_cookie
    elseif err then
        ngx.log(ngx.ERR, "failed to get jasima_pool cookie: ", err)
    end

    return nil
end

function _M.load_pool_mapping(pool_name)
    if not pool_name then pool_name = "public" end

    local cache = ngx.shared.jasima_cache
    local cache_key = "poolmap:" .. pool_name
    local cached = cache:get(cache_key)
    if cached then return cjson.decode(cached) end

    local red = redis:new()
    red:set_timeout(1000)
    local ok, err = red:connect("redis", 6379)
    if not ok then return nil, "Redis connect failed: " .. err end

    local key = "jasima:poolmap:" .. pool_name
    local res, err = red:get(key)
    if not res or res == ngx.null then return nil, "No pool mapping found" end

    red:set_keepalive(10000, 100)
    cache:set(cache_key, res, 60)

    return cjson.decode(res)
end

function _M.load_config(jasima_host)
    local cache = ngx.shared.jasima_cache
    local cache_key = "config:" .. jasima_host
    local cached = cache:get(cache_key)
    if cached then return cjson.decode(cached) end

    local red = redis:new()
    red:set_timeout(1000)
    local ok, err = red:connect("redis", 6379)
    if not ok then return nil, "Redis connect failed: " .. err end

    local key = "jasima:config:" .. jasima_host
    local res, err = red:get(key)
    if not res or res == ngx.null then return nil, "No config in Redis" end

    red:set_keepalive(10000, 100)

    cache:set(cache_key, res, 60)
    return cjson.decode(res)
end

return _M
