local resolver = require "resty.dns.resolver"
local iputils = require "resty.iputils"

iputils.enable_lrucache()

local bogon_ips = iputils.parse_cidrs({
    "0.0.0.0/8",
    "10.0.0.0/8",
    "100.64.0.0/10",
    "127.0.0.0/8",
    "169.254.0.0/16",
    "172.16.0.0/12",
    "192.0.0.0/24",
    "192.0.2.0/24",
    "192.168.0.0/16",
    "198.18.0.0/15",
    "198.51.100.0/24",
    "203.0.113.0/24",
    "224.0.0.0/4",
    "240.0.0.0/4"
})

local _M = {}

function _M.resolve_origin(origin_host)
    local cache = ngx.shared.jasima_cache
    local cache_key = "upstream_ips:" .. origin_host
    local cached = cache:get(cache_key)
    if cached then return cached end

    local r, err = resolver:new{
        nameservers = {"8.8.8.8", "8.8.4.4"},
        retrans = 5,
        timeout = 2000,
    }

    if not r then
        ngx.log(ngx.ERR, "failed to instantiate resolver: ", err)
        return ngx.exit(500)
    end

    local answers, err = r:query(origin_host, {qtype = r.TYPE_A})
    if not answers then
        ngx.log(ngx.ERR, "failed to query: ", err)
        return ngx.exit(500)
    end

    if answers.errcode then
        ngx.log(ngx.ERR, "DNS error code: ", answers.errcode, ": ", answers.errstr)
        return ngx.exit(500)
    end

    local origin_ips = {}
    for _, ans in pairs(answers) do
        if ans.address then
            table.insert(origin_ips, ans.address)
        end
    end

    cache:set(cache_key, origin_ips, 60)

    return origin_ips
end

function _M.filter_bogons(ip_list)
    local filtered = {}
    for _, ip in ipairs(ip_list) do
        if not iputils.ip_in_cidrs(ip, bogon_ips) then
            table.insert(filtered, ip)
        end
    end
    return filtered
end

function _M.get_mirror(host)
    ngx.log(ngx.DEBUG, "Looking up mirror for " .. host)
    local host_lower = host:lower()
    local mirror = ngx.ctx.jasima_pool_map[host_lower]
    return "//" .. (mirror or host) .. "/"
end

return _M
